"""A simple "symbol" class for Python.  Symbols are like strings,
except that symbols should only be comparable to other symbols.  This
module is designed so that one should be able to do a "from symbol
import *" on this.

To create a new Symbol, use the Symbol() function, like this:

    hello = Symbol("hello")

All symbols are interned so that comparison using 'is' will work.  For
example,

    Symbol("hello") is Symbol("h" + "ello")

should work.
"""

from UserString import UserString as __UserString

class __Symbol(__UserString):
    """Support class for symbols."""
    def __eq__(self, other):
        """Comparison should only be possible between symbols."""
        if not isSymbol(other): return 0
        return self.data == other.data


"""A global hashtable that contains all known symbols."""
__interned_symbols = {}
def Symbol(s):
    global __interned_symbols
    s = s.upper()
    if not __interned_symbols.has_key(s):
        __interned_symbols[s] = __Symbol(s)
    return __interned_symbols[s]


"""Here are definitions of symbols that we should know about."""
false = Symbol("false")
true = Symbol("true")
__empty_symbol = Symbol("")

def isSymbol(x):
    return type(x) == type(__empty_symbol)
